@@ -54,7 +54,7 @@ other modules:
 
     get_contents()
         Fetches the "contents" of an Action for signature calculation
-        plus the varlist.  This is what gets MD5 checksummed to decide
+        plus the varlist.  This is what gets checksummed to decide
         if a target needs to be rebuilt because its action changed.
 
     genstring()
@@ -106,7 +106,6 @@ import re
 import sys
 import subprocess
 from subprocess import DEVNULL
-import itertools
 import inspect
 from collections import OrderedDict
 
@@ -122,9 +121,9 @@ from SCons.Util import is_String, is_List
 class _null:
     pass
 
-print_actions = 1
-execute_actions = 1
-print_actions_presub = 0
+print_actions = True
+execute_actions = True
+print_actions_presub = False
 
 # Use pickle protocol 1 when pickling functions for signature
 # otherwise python3 and python2 will yield different pickles
@@ -145,7 +144,7 @@ def rfile(n):
 def default_exitstatfunc(s):
     return s
 
-strip_quotes = re.compile('^[\'"](.*)[\'"]$')
+strip_quotes = re.compile(r'^[\'"](.*)[\'"]$')
 
 
 def _callable_contents(obj):
@@ -756,6 +755,29 @@ def get_default_ENV(env):
         return default_ENV
 
 
+def _resolve_shell_env(env, target, source):
+    """
+    First get default environment.
+    Then if SHELL_ENV_GENERATORS is set and is iterable,
+    call each callable in that list to allow it to alter
+    the created execution environment.
+    """
+    ENV = get_default_ENV(env)
+    shell_gen = env.get('SHELL_ENV_GENERATORS')
+    if shell_gen:
+        try:
+            shell_gens = iter(shell_gen)
+        except TypeError:
+            raise SCons.Errors.UserError("SHELL_ENV_GENERATORS must be iteratable.")
+        else:
+            ENV = ENV.copy()
+            for generator in shell_gens:
+                ENV = generator(env, target, source, ENV)
+                if not isinstance(ENV, dict):
+                    raise SCons.Errors.UserError(f"SHELL_ENV_GENERATORS function: {generator} must return a dict.")
+    return ENV
+
+
 def _subproc(scons_env, cmd, error='ignore', **kw):
     """Wrapper for subprocess which pulls from construction env.
 
@@ -800,9 +822,25 @@ def _subproc(scons_env, cmd, error='ignore', **kw):
         if error == 'raise': raise
         # return a dummy Popen instance that only returns error
         class dummyPopen:
-            def __init__(self, e): self.exception = e
-            def communicate(self, input=None): return ('', '')
-            def wait(self): return -self.exception.errno
+            def __init__(self, e):
+                self.exception = e
+            # Add the following two to enable using the return value as a context manager
+            # for example
+            #    with Action._subproc(...) as po:
+            #       logic here which uses po
+
+            def __enter__(self):
+                return self
+
+            def __exit__(self, *args):
+                pass
+
+            def communicate(self, input=None):
+                return ('', '')
+
+            def wait(self):
+                return -self.exception.errno
+
             stdin = None
             class f:
                 def read(self): return ''
@@ -833,7 +871,7 @@ class CommandAction(_ActionAction):
         # variables.
         if SCons.Debug.track_instances: logInstanceCreation(self, 'Action.CommandAction')
 
-        _ActionAction.__init__(self, **kw)
+        super().__init__(**kw)
         if is_List(cmd):
             if [c for c in cmd if is_List(c)]:
                 raise TypeError("CommandAction should be given only "
@@ -909,7 +947,7 @@ class CommandAction(_ActionAction):
 
         escape = env.get('ESCAPE', lambda x: x)
 
-        ENV = get_default_ENV(env)
+        ENV = _resolve_shell_env(env, target, source)
 
         # Ensure that the ENV values are all strings:
         for key, value in ENV.items():
@@ -968,7 +1006,7 @@ class CommandAction(_ActionAction):
         if is_String(icd) and icd[:1] == '$':
             icd = env.subst(icd)
 
-        if not icd or str(icd).lower in ('0', 'none', 'false', 'no', 'off'):
+        if not icd or str(icd).lower() in ('0', 'none', 'false', 'no', 'off'):
             return []
 
         try:
@@ -976,7 +1014,7 @@ class CommandAction(_ActionAction):
         except ValueError:
             icd_int = None
 
-        if (icd_int and icd_int > 1) or icd == 'all':
+        if (icd_int and icd_int > 1) or str(icd).lower() == 'all':
             # An integer value greater than 1 specifies the number of entries
             # to scan. "all" means to scan all.
             return self._get_implicit_deps_heavyweight(target, source, env, executor, icd_int)
@@ -1216,7 +1254,7 @@ class FunctionAction(_ActionAction):
                 # This is weird, just do the best we can.
                 self.funccontents = _object_contents(execfunction)
 
-        _ActionAction.__init__(self, **kw)
+        super().__init__(**kw)
 
     def function_name(self):
         try:
