@@ -1,18 +1,6 @@
-"""SCons.SConf
-
-Autoconf-like configuration support.
-
-In other words, SConf allows to run tests on the build machine to detect
-capabilities of system and do some things based on result: generate config
-files, header files for C/C++, update variables in environment.
-
-Tests on the build system can detect if compiler sees header files, if
-libraries are installed, if some command line options are supported etc.
-
-"""
-
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -32,13 +20,20 @@ libraries are installed, if some command line options are supported etc.
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
-from __future__ import print_function
 
-__revision__ = "/src/script/src/engine/SCons/SConf.py none 2019-12-16 19:06:27 none"
+"""Autoconf-like configuration support.
+
+In other words, SConf allows to run tests on the build machine to detect
+capabilities of system and do some things based on result: generate config
+files, header files for C/C++, update variables in environment.
+
+Tests on the build system can detect if compiler sees header files, if
+libraries are installed, if some command line options are supported etc.
+"""
 
 import SCons.compat
 
+import atexit
 import io
 import os
 import re
@@ -56,6 +51,7 @@ import SCons.Warnings
 import SCons.Conftest
 
 from SCons.Debug import Trace
+from collections import defaultdict
 
 # Turn off the Conftest error logging
 SCons.Conftest.LogInputFiles = 0
@@ -65,9 +61,9 @@ SCons.Conftest.LogErrorMessages = 0
 build_type = None
 build_types = ['clean', 'help']
 
-def SetBuildType(type):
+def SetBuildType(buildtype):
     global build_type
-    build_type = type
+    build_type = buildtype
 
 # to be set, if we are in dry-run mode
 dryrun = 0
@@ -77,6 +73,9 @@ FORCE=1 # force all tests to be rebuilt
 CACHE=2 # force all tests to be taken from cache (raise an error, if necessary)
 cache_mode = AUTO
 
+def _set_conftest_node(node):
+    node.attributes.conftest_node = 1
+
 def SetCacheMode(mode):
     """Set the Configure cache mode. mode must be one of "auto", "force",
     or "cache"."""
@@ -98,7 +97,7 @@ def SetProgressDisplay(display):
 
 SConfFS = None
 
-_ac_build_counter = 0 # incremented, whenever TryBuild is called
+_ac_build_counter = defaultdict(int)
 _ac_config_logs = {}  # all config.log files created in this build
 _ac_config_hs   = {}  # all config.h files created in this build
 sconf_global = None   # current sconf object
@@ -132,18 +131,18 @@ def CreateConfigHBuilder(env):
                                  _stringConfigH)
     sconfigHBld = SCons.Builder.Builder(action=action)
     env.Append( BUILDERS={'SConfigHBuilder':sconfigHBld} )
-    for k in list(_ac_config_hs.keys()):
-        env.SConfigHBuilder(k, env.Value(_ac_config_hs[k]))
+    for k, v in _ac_config_hs.items():
+        env.SConfigHBuilder(k, env.Value(v))
 
 
-class SConfWarning(SCons.Warnings.Warning):
+class SConfWarning(SCons.Warnings.SConsWarning):
     pass
 SCons.Warnings.enableWarningClass(SConfWarning)
 
 # some error definitions
 class SConfError(SCons.Errors.UserError):
     def __init__(self,msg):
-        SCons.Errors.UserError.__init__(self,msg)
+        super().__init__(msg)
 
 class ConfigureDryRunError(SConfError):
     """Raised when a file or directory needs to be updated during a Configure
@@ -153,19 +152,22 @@ class ConfigureDryRunError(SConfError):
             msg = 'Cannot create configure directory "%s" within a dry-run.' % str(target)
         else:
             msg = 'Cannot update configure test "%s" within a dry-run.' % str(target)
-        SConfError.__init__(self,msg)
+        super().__init__(msg)
 
 class ConfigureCacheError(SConfError):
     """Raised when a use explicitely requested the cache feature, but the test
     is run the first time."""
     def __init__(self,target):
-        SConfError.__init__(self, '"%s" is not yet built and cache is forced.' % str(target))
+        super().__init__('"%s" is not yet built and cache is forced.' % str(target))
+
 
 # define actions for building text files
-def _createSource( target, source, env ):
+def _createSource(target, source, env):
     fd = open(str(target[0]), "w")
     fd.write(source[0].get_contents().decode())
     fd.close()
+
+
 def _stringSource( target, source, env ):
     return (str(target[0]) + ' <-\n  |' +
             source[0].get_contents().decode().replace( '\n', "\n  |" ) )
@@ -187,7 +189,7 @@ class SConfBuildInfo(SCons.Node.FS.FileBuildInfo):
         self.string = string
 
 
-class Streamer(object):
+class Streamer:
     """
     'Sniffer' for a file-like writable object. Similar to the unix tool tee.
     """
@@ -235,8 +237,10 @@ class SConfBuildTask(SCons.Taskmaster.AlwaysTask):
         bi.
         """
         if not isinstance(bi, SConfBuildInfo):
-            SCons.Warnings.warn(SConfWarning,
-              "The stored build information has an unexpected class: %s" % bi.__class__)
+            SCons.Warnings.warn(
+                SConfWarning,
+                "The stored build information has an unexpected class: %s" % bi.__class__
+            )
         else:
             self.display("The original builder output was:\n" +
                          ("  |" + str(bi.string)).replace("\n", "\n  |"))
@@ -270,7 +274,7 @@ class SConfBuildTask(SCons.Taskmaster.AlwaysTask):
         cached_error = False
         cachable = True
         for t in self.targets:
-            if T: Trace('%s' % (t))
+            if T: Trace('%s' % t)
             bi = t.get_stored_info().binfo
             if isinstance(bi, SConfBuildInfo):
                 if T: Trace(': SConfBuildInfo')
@@ -280,7 +284,7 @@ class SConfBuildTask(SCons.Taskmaster.AlwaysTask):
                 else:
                     if T: Trace(': get_state() %s' % t.get_state())
                     if T: Trace(': changed() %s' % t.changed())
-                    if (t.get_state() != SCons.Node.up_to_date and t.changed()):
+                    if t.get_state() != SCons.Node.up_to_date and t.changed():
                         changed = True
                     if T: Trace(': changed %s' % changed)
                 cached_error = cached_error or bi.result
@@ -372,7 +376,7 @@ class SConfBuildTask(SCons.Taskmaster.AlwaysTask):
                     sconsign.set_entry(t.name, sconsign_entry)
                     sconsign.merge()
 
-class SConfBase(object):
+class SConfBase:
     """This is simply a class to represent a configure context. After
     creating a SConf object, you can call any tests. After finished with your
     tests, be sure to call the Finish() method, which returns the modified
@@ -426,7 +430,8 @@ class SConfBase(object):
             SConfFS = SCons.Node.FS.default_fs or \
                       SCons.Node.FS.FS(env.fs.pathTop)
         if sconf_global is not None:
-            raise SCons.Errors.UserError
+            raise SCons.Errors.UserError("""Configure() called while another Configure() exists.
+            Please call .Finish() before creating and second Configure() context""")
 
         if log_file is not None:
             log_file = SConfFS.File(env.subst(log_file))
@@ -445,6 +450,7 @@ class SConfBase(object):
                  'CheckFunc'          : CheckFunc,
                  'CheckType'          : CheckType,
                  'CheckTypeSize'      : CheckTypeSize,
+                 'CheckMember'        : CheckMember,
                  'CheckDeclaration'   : CheckDeclaration,
                  'CheckHeader'        : CheckHeader,
                  'CheckCHeader'       : CheckCHeader,
@@ -516,6 +522,7 @@ class SConfBase(object):
         # we override the store_info() method with a null place-holder
         # so we really control how it gets written.
         for n in nodes:
+            _set_conftest_node(n)
             n.store_info = 0
             if not hasattr(n, 'attributes'):
                 n.attributes = SCons.Node.Node.Attrs()
@@ -528,6 +535,7 @@ class SConfBase(object):
                 for c in n.children(scan=False):
                     # Keep debug code here.
                     # print("Checking [%s] for builders and then setting keep_targetinfo"%c)
+                    _set_conftest_node(c)
                     if  c.has_builder():
                         n.store_info = 0
                         if not hasattr(c, 'attributes'):
@@ -573,8 +581,7 @@ class SConfBase(object):
         """
         return self.pspawn(sh, escape, cmd, args, env, self.logstream, self.logstream)
 
-
-    def TryBuild(self, builder, text = None, extension = ""):
+    def TryBuild(self, builder, text=None, extension=""):
         """Low level TryBuild implementation. Normally you don't need to
         call that - you can use TryCompile / TryLink / TryRun instead
         """
@@ -592,28 +599,44 @@ class SConfBase(object):
             raise SCons.Errors.UserError('Missing SPAWN construction variable.')
 
         nodesToBeBuilt = []
+        sourcetext = self.env.Value(text)
+        _set_conftest_node(sourcetext)
+        f = "conftest"
+
+        if text is not None:
+            textSig = SCons.Util.hash_signature(sourcetext)
+            textSigCounter = str(_ac_build_counter[textSig])
+            _ac_build_counter[textSig] += 1
+
+            f = "_".join([f, textSig, textSigCounter])
+            textFile = self.confdir.File(f + extension)
+            _set_conftest_node(textFile)
+            textFileNode = self.env.SConfSourceBuilder(target=textFile,
+                                                       source=sourcetext)
+            nodesToBeBuilt.extend(textFileNode)
+
+            source = textFile
+            target = textFile.File(f + "SConfActionsContentDummyTarget")
+            _set_conftest_node(target)
+        else:
+            source = None
+            target = None
+
+        action = builder.builder.action.get_contents(target=target, source=[source], env=self.env)
+        actionsig = SCons.Util.hash_signature(action)
+        f = "_".join([f, actionsig])
 
-        f = "conftest_" + str(_ac_build_counter)
         pref = self.env.subst( builder.builder.prefix )
         suff = self.env.subst( builder.builder.suffix )
         target = self.confdir.File(pref + f + suff)
+        _set_conftest_node(target)
 
         try:
             # Slide our wrapper into the construction environment as
             # the SPAWN function.
             self.env['SPAWN'] = self.pspawn_wrapper
-            sourcetext = self.env.Value(text)
 
-            if text is not None:
-                textFile = self.confdir.File(f + extension)
-                textFileNode = self.env.SConfSourceBuilder(target=textFile,
-                                                           source=sourcetext)
-                nodesToBeBuilt.extend(textFileNode)
-                source = textFileNode
-            else:
-                source = None
-
-            nodes = builder(target = target, source = source)
+            nodes = builder(target = target, source = source, SCONF_NODE=True)
             if not SCons.Util.is_List(nodes):
                 nodes = [nodes]
             nodesToBeBuilt.extend(nodes)
@@ -622,7 +645,6 @@ class SConfBase(object):
         finally:
             self.env['SPAWN'] = save_spawn
 
-        _ac_build_counter = _ac_build_counter + 1
         if result:
             self.lastTarget = nodes[0]
         else:
@@ -668,7 +690,7 @@ class SConfBase(object):
         is saved in self.lastTarget (for further processing).
         """
         ok = self.TryLink(text, extension)
-        if( ok ):
+        if ok:
             prog = self.lastTarget
             pname = prog.get_internal_path()
             output = self.confdir.File(os.path.basename(pname)+'.out')
@@ -679,7 +701,7 @@ class SConfBase(object):
                 return( 1, outputStr)
         return (0, "")
 
-    class TestWrapper(object):
+    class TestWrapper:
         """A wrapper around Tests (to ensure sanity)"""
         def __init__(self, test, sconf):
             self.test = test
@@ -703,7 +725,7 @@ class SConfBase(object):
         """Adds all the tests given in the tests dictionary to this SConf
         instance
         """
-        for name in list(tests.keys()):
+        for name in tests.keys():
             self.AddTest(name, tests[name])
 
     def _createDir( self, node ):
@@ -737,11 +759,16 @@ class SConfBase(object):
                 _ac_config_logs[self.logfile] = None
                 log_mode = "w"
             fp = open(str(self.logfile), log_mode)
+
+            def conflog_cleanup(logf):
+                logf.close()
+
+            atexit.register(conflog_cleanup, fp)
             self.logstream = SCons.Util.Unbuffered(fp)
             # logfile may stay in a build directory, so we tell
-            # the build system not to override it with a eventually
+            # the build system not to override it with an eventually
             # existing file with the same name in the source directory
-            self.logfile.dir.add_ignore( [self.logfile] )
+            self.logfile.dir.add_ignore([self.logfile])
 
             tb = traceback.extract_stack()[-3-self.depth]
             old_fs_dir = SConfFS.getcwd()
@@ -789,7 +816,7 @@ class SConfBase(object):
             _ac_config_hs[self.config_h] = self.config_h_text
         self.env.fs = self.lastEnvFs
 
-class CheckContext(object):
+class CheckContext:
     """Provides a context for configure tests. Defines how a test writes to the
     screen and log file.
 
@@ -866,9 +893,9 @@ class CheckContext(object):
         return self.sconf.TryRun(*args, **kw)
 
     def __getattr__( self, attr ):
-        if( attr == 'env' ):
+        if attr == 'env':
             return self.sconf.env
-        elif( attr == 'lastTarget' ):
+        elif attr == 'lastTarget':
             return self.sconf.lastTarget
         else:
             raise AttributeError("CheckContext instance has no attribute '%s'" % attr)
@@ -966,6 +993,13 @@ def CheckDeclaration(context, declaration, includes = "", language = None):
     context.did_show_result = 1
     return not res
 
+def CheckMember(context, aggregate_member, header = None, language = None):
+    '''Returns the status (False : failed, True : ok).'''
+    res = SCons.Conftest.CheckMember(context, aggregate_member, header=header, language=language)
+    context.did_show_result = 1
+    return not res
+
+
 def createIncludesFromHeaders(headers, leaveLast, include_quotes = '""'):
     # used by CheckHeader and CheckLibWithHeader to produce C - #include
     # statements from the specified header (list)
@@ -1067,7 +1101,7 @@ def CheckLibWithHeader(context, libs, header, language,
     """
     prog_prefix, dummy = \
                  createIncludesFromHeaders(header, 0)
-    if libs == []:
+    if not libs:
         libs = [None]
 
     if not SCons.Util.is_List(libs):
