@@ -1,4 +1,4 @@
-"""engine.SCons.Tool.aixf77
+"""SCons.Tool.aixf77
 
 Tool-specific initialization for IBM Visual Age f77 Fortran compiler.
 
@@ -8,7 +8,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# __COPYRIGHT__
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,7 +30,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/src/script/src/engine/SCons/Tool/aixf77.py none 2019-12-16 19:06:27 none"
+__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 
 import os.path
 
