@@ -1,15 +1,6 @@
-"""SCons.Tool.msvs
-
-Tool-specific initialization for Microsoft Visual Studio project files.
-
-There normally shouldn't be any need to import this module directly.
-It will usually be imported through the generic SCons.Tool.Tool()
-selection method.
-
-"""
-
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,14 +21,15 @@ selection method.
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import print_function
+""" Tool-specific initialization for Microsoft Visual Studio project files.
 
-__revision__ = "/src/script/src/engine/SCons/Tool/msvs.py none 2019-12-16 19:06:27 none"
-
-import SCons.compat
+There normally shouldn't be any need to import this module directly.
+It will usually be imported through the generic SCons.Tool.Tool()
+selection method.
+"""
 
 import base64
-import hashlib
+import uuid
 import ntpath
 import os
 import pickle
@@ -51,10 +43,11 @@ import SCons.Script.SConscript
 import SCons.PathList
 import SCons.Util
 import SCons.Warnings
-
-from .MSCommon import msvc_exists, msvc_setup_env_once
 from SCons.Defaults import processDefines
 from SCons.compat import PICKLE_PROTOCOL
+from .MSCommon import msvc_setup_env_tool, msvc_setup_env_once
+
+tool_name = 'msvs'
 
 ##############################################################################
 # Below here are the classes and functions for generation of
@@ -70,6 +63,7 @@ def xmlify(s):
     s = s.replace('\n', '&#x0A;')
     return s
 
+
 def processIncludes(includes, env, target, source):
     """
     Process a CPPPATH list in includes, given the env, target and source.
@@ -79,26 +73,57 @@ def processIncludes(includes, env, target, source):
     return [env.Dir(i).abspath for i in
             SCons.PathList.PathList(includes).subst_path(env, target, source)]
 
+# Work-in-progress
+# individual elements each get a globally unique identifier.
+# however, there also are some "well known" guids that either
+# represent something, or are used as a namespace to base
+# generating guids that should be "in" the namespace
+#NAMESPACE_PROJECT   = uuid.UUID("{D9BD5916-F055-4D77-8C69-9448E02BF433}")
+#NAMESPACE_SLN_GROUP = uuid.UUID("{2D0C29E0-512F-47BE-9AC4-F4CAE74AE16E}")
+#NAMESPACE_INTERNAL  = uuid.UUID("{BAA4019E-6D67-4EF1-B3CB-AE6CD82E4060}")
+
+# Kinds of projects, as used in solution files
+#PROJECT_KIND_C      = "{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}"
+#PROJECT_KIND_PCL    = "{786C830F-07A1-408B-BD7F-6EE04809D6DB}"
+#NAMESPACE_PRJ_FOLDER = "{66A26720-8FB5-11D2-AA7E-00C04F688DDE}"
+#NAMESPACE_SLN_FOLDER = "{2150E333-8FDC-42A3-9474-1A3956D46DE8}"
+#NAMESPACE_TEST       = "{3AC096D0-A1C2-E12C-1390-A8335801FDAB}"
+
+def processFlags(flags, env):
+    """
+    If /std:c++XX is in flags then we need to ensure /Zc:__cplusplus is in
+    flags to tell intellisense to respect our specified standard
+    """
+    if any(f.startswith('/std:c++') for f in flags) and \
+       not any(f == '/Zc:__cplusplus' for f in flags):
+        flags.append('/Zc:__cplusplus')
+    return [env.subst(f) for f in flags]
+
 
 external_makefile_guid = '{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}'
 
-def _generateGUID(slnfile, name):
-    """This generates a dummy GUID for the sln file to use.  It is
-    based on the MD5 signatures of the sln filename plus the name of
-    the project.  It basically just needs to be unique, and not
-    change with each invocation."""
-    m = hashlib.md5()
+def _generateGUID(slnfile, name, namespace=external_makefile_guid):
+    """Generates a GUID for the sln file to use.
+
+    The uuid5 function is used to combine an existing namespace uuid -
+    the one VS uses for C projects (external_makedile_guid) -
+    and a combination of the solution file name (slnfile) and the
+    project name (name).  We just need uniqueness/repeatability.
+
+    Returns (str) "displayable" GUID, already wrapped in braces
+    """
+
     # Normalize the slnfile path to a Windows path (\ separators) so
     # the generated file has a consistent GUID even if we generate
     # it on a non-Windows platform.
-    m.update(bytearray(ntpath.normpath(str(slnfile)) + str(name),'utf-8'))
-    solution = m.hexdigest().upper()
-    # convert most of the signature to GUID form (discard the rest)
-    solution = "{" + solution[:8] + "-" + solution[8:12] + "-" + solution[12:16] + "-" + solution[16:20] + "-" + solution[20:32] + "}"
-    return solution
+    slnfile = ntpath.normpath(str(slnfile))
+    solution = uuid.uuid5(uuid.UUID(namespace), "%s-%s" %(slnfile, name))
+    return '{' + str(solution).upper() + '}'
+
 
 version_re = re.compile(r'(\d+\.\d+)(.*)')
 
+
 def msvs_parse_version(s):
     """
     Split a Visual Studio version, which may in fact be something like
@@ -141,7 +166,7 @@ else:
     python_executable = os.path.join('$$(PYTHON_ROOT)',
                                      os.path.split(sys.executable)[1])
 
-class Config(object):
+class Config:
     pass
 
 def splitFully(path):
@@ -171,10 +196,10 @@ def makeHierarchy(sources):
         #    print 'Warning: failed to decompose path for '+str(file)
     return hierarchy
 
-class _UserGenerator(object):
-    '''
+class _UserGenerator:
+    """
     Base class for .dsp.user file generator
-    '''
+    """
     # Default instance values.
     # Ok ... a bit defensive, but it does not seem reasonable to crash the
     # build for a workspace user file. :-)
@@ -220,7 +245,7 @@ class _UserGenerator(object):
             for var, src in dbg_settings.items():
                 # Update only expected keys
                 trg = {}
-                for key in [k for k in list(self.usrdebg.keys()) if k in src]:
+                for key in [k for k in self.usrdebg.keys() if k in src]:
                     trg[key] = str(src[key])
                 self.configs[var].debug = trg
 
@@ -297,7 +322,7 @@ class _GenerateV7User(_UserGenerator):
             self.usrhead = V9UserHeader
             self.usrconf = V9UserConfiguration
             self.usrdebg = V9DebugSettings
-        _UserGenerator.__init__(self, dspfile, source, env)
+        super().__init__(dspfile, source, env)
 
     def UserProject(self):
         confkeys = sorted(self.configs.keys())
@@ -356,7 +381,10 @@ class _GenerateV10User(_UserGenerator):
 
     def __init__(self, dspfile, source, env):
         version_num, suite = msvs_parse_version(env['MSVS_VERSION'])
-        if version_num >= 14.2:
+        if version_num >= 14.3:
+            # Visual Studio 2022 is considered to be version 17.
+            self.versionstr = '17.0'
+        elif version_num >= 14.2:
             # Visual Studio 2019 is considered to be version 16.
             self.versionstr = '16.0'
         elif version_num >= 14.1:
@@ -369,7 +397,7 @@ class _GenerateV10User(_UserGenerator):
         self.usrhead = V10UserHeader
         self.usrconf = V10UserConfiguration
         self.usrdebg = V10DebugSettings
-        _UserGenerator.__init__(self, dspfile, source, env)
+        super().__init__(dspfile, source, env)
 
     def UserProject(self):
         confkeys = sorted(self.configs.keys())
@@ -384,7 +412,7 @@ class _GenerateV10User(_UserGenerator):
                 self.usrfile.write(self.usrconf % locals())
         self.usrfile.write('</Project>')
 
-class _DSPGenerator(object):
+class _DSPGenerator:
     """ Base class for DSP generators """
 
     srcargs = [
@@ -511,6 +539,10 @@ class _DSPGenerator(object):
             cpppaths = GetKeyFromEnv(env, 'cpppaths', variants)
         else:
             cpppaths = [env.get('CPPPATH', [])] * len(variants)
+        if 'cppflags' in env:
+            cppflags = GetKeyFromEnv(env, 'cppflags', variants)
+        else:
+            cppflags = [env.get('CCFLAGS', []) + env.get('CXXFLAGS', []) + env.get('CPPFLAGS', [])] * len(variants)
 
         self.env = env
 
@@ -544,7 +576,7 @@ class _DSPGenerator(object):
             if t[1] in self.env:
                 if SCons.Util.is_List(self.env[t[1]]):
                     for i in self.env[t[1]]:
-                        if not i in self.sources[t[0]]:
+                        if i not in self.sources[t[0]]:
                             self.sources[t[0]].append(i)
                 else:
                     if not self.env[t[1]] in self.sources[t[0]]:
@@ -553,12 +585,13 @@ class _DSPGenerator(object):
         for n in sourcenames:
             self.sources[n].sort(key=lambda a: a.lower())
 
-        def AddConfig(self, variant, buildtarget, outdir, runfile, cmdargs, cppdefines, cpppaths, dspfile=dspfile, env=env):
+        def AddConfig(self, variant, buildtarget, outdir, runfile, cmdargs, cppdefines, cpppaths, cppflags, dspfile=dspfile, env=env):
             config = Config()
             config.buildtarget = buildtarget
             config.outdir = outdir
             config.cmdargs = cmdargs
             config.cppdefines = cppdefines
+            config.cppflags = cppflags
             config.runfile = runfile
 
             # Dir objects can't be pickled, so we need an absolute path here.
@@ -573,16 +606,16 @@ class _DSPGenerator(object):
                 config.platform = 'Win32'
 
             self.configs[variant] = config
+            # DEBUG: leave enabled, test/MSVS/CPPPATH-dirs.py expects this
             print("Adding '" + self.name + ' - ' + config.variant + '|' + config.platform + "' to '" + str(dspfile) + "'")
 
         for i in range(len(variants)):
-            AddConfig(self, variants[i], buildtarget[i], outdir[i], runfile[i], cmdargs[i], cppdefines[i], cpppaths[i])
+            AddConfig(self, variants[i], buildtarget[i], outdir[i], runfile[i], cmdargs[i], cppdefines[i], cpppaths[i], cppflags[i])
+
+        seen = set()
+        self.platforms = [p.platform for p in self.configs.values()
+                          if not (p.platform in seen or seen.add(p.platform))]
 
-        self.platforms = []
-        for key in list(self.configs.keys()):
-            platform = self.configs[key].platform
-            if platform not in self.platforms:
-                self.platforms.append(platform)
 
     def Build(self):
         pass
@@ -702,7 +735,7 @@ class _GenerateV6DSP(_DSPGenerator):
                       'Resource Files': 'r|rc|ico|cur|bmp|dlg|rc2|rct|bin|cnt|rtf|gif|jpg|jpeg|jpe',
                       'Other Files': ''}
 
-        for kind in sorted(list(categories.keys()), key=lambda a: a.lower()):
+        for kind in sorted(categories.keys(), key=lambda a: a.lower()):
             if not self.sources[kind]:
                 continue # skip empty groups
 
@@ -748,7 +781,7 @@ class _GenerateV6DSP(_DSPGenerator):
             data = pickle.loads(datas)
         except KeyboardInterrupt:
             raise
-        except:
+        except Exception:
             return # unable to unpickle any data for some reason
 
         self.configs.update(data)
@@ -980,7 +1013,7 @@ class _GenerateV7DSP(_DSPGenerator, _GenerateV7User):
             if SCons.Util.is_Dict(value):
                 self.file.write('\t\t\t<Filter\n'
                                 '\t\t\t\tName="%s"\n'
-                                '\t\t\t\tFilter="">\n' % (key))
+                                '\t\t\t\tFilter="">\n' % key)
                 self.printSources(value, commonprefix)
                 self.file.write('\t\t\t</Filter>\n')
 
@@ -992,7 +1025,7 @@ class _GenerateV7DSP(_DSPGenerator, _GenerateV7User):
                 file = os.path.normpath(file)
                 self.file.write('\t\t\t<File\n'
                                 '\t\t\t\tRelativePath="%s">\n'
-                                '\t\t\t</File>\n' % (file))
+                                '\t\t\t</File>\n' % file)
 
     def PrintSourceFiles(self):
         categories = {'Source Files': 'cpp;c;cxx;l;y;def;odl;idl;hpj;bat',
@@ -1003,7 +1036,7 @@ class _GenerateV7DSP(_DSPGenerator, _GenerateV7User):
 
         self.file.write('\t<Files>\n')
 
-        cats = sorted([k for k in list(categories.keys()) if self.sources[k]],
+        cats = sorted([k for k in categories.keys() if self.sources[k]],
                       key=lambda a: a.lower())
         for kind in cats:
             if len(cats) > 1:
@@ -1148,6 +1181,7 @@ V10DSPCommandLine = """\
 \t\t<NMakeForcedIncludes Condition="'$(Configuration)|$(Platform)'=='%(variant)s|%(platform)s'">$(NMakeForcedIncludes)</NMakeForcedIncludes>
 \t\t<NMakeAssemblySearchPath Condition="'$(Configuration)|$(Platform)'=='%(variant)s|%(platform)s'">$(NMakeAssemblySearchPath)</NMakeAssemblySearchPath>
 \t\t<NMakeForcedUsingAssemblies Condition="'$(Configuration)|$(Platform)'=='%(variant)s|%(platform)s'">$(NMakeForcedUsingAssemblies)</NMakeForcedUsingAssemblies>
+\t\t<AdditionalOptions Condition="'$(Configuration)|$(Platform)'=='%(variant)s|%(platform)s'">%(additionaloptions)s</AdditionalOptions>
 """
 
 V15DSPHeader = """\
@@ -1247,6 +1281,7 @@ class _GenerateV10DSP(_DSPGenerator, _GenerateV10User):
             cmdargs = self.configs[kind].cmdargs
             cpppaths = self.configs[kind].cpppaths
             cppdefines = self.configs[kind].cppdefines
+            cppflags = self.configs[kind].cppflags
 
             env_has_buildtarget = 'MSVSBUILDTARGET' in self.env
             if not env_has_buildtarget:
@@ -1266,6 +1301,7 @@ class _GenerateV10DSP(_DSPGenerator, _GenerateV10User):
             # assumes they don't.
             preprocdefs = xmlify(';'.join(processDefines(cppdefines)))
             includepath = xmlify(';'.join(processIncludes(cpppaths, self.env, None, None)))
+            additionaloptions = xmlify(' '.join(processFlags(cppflags, self.env)))
 
             if not env_has_buildtarget:
                 del self.env['MSVSBUILDTARGET']
@@ -1348,7 +1384,7 @@ class _GenerateV10DSP(_DSPGenerator, _GenerateV10User):
                       'Resource Files': 'r;rc;ico;cur;bmp;dlg;rc2;rct;bin;cnt;rtf;gif;jpg;jpeg;jpe',
                       'Other Files': ''}
 
-        cats = sorted([k for k in list(categories.keys()) if self.sources[k]],
+        cats = sorted([k for k in categories.keys() if self.sources[k]],
                       key = lambda a: a.lower())
 
         # print vcxproj.filters file first
@@ -1408,7 +1444,9 @@ class _GenerateV10DSP(_DSPGenerator, _GenerateV10User):
                         '\t</ItemGroup>\n' % str(self.sconscript))
 
     def Parse(self):
-        print("_GenerateV10DSP.Parse()")
+        # DEBUG
+        # print("_GenerateV10DSP.Parse()")
+        pass
 
     def Build(self):
         try:
@@ -1422,7 +1460,7 @@ class _GenerateV10DSP(_DSPGenerator, _GenerateV10User):
 
         _GenerateV10User.Build(self)
 
-class _DSWGenerator(object):
+class _DSWGenerator:
     """ Base class for DSW generators """
     def __init__(self, dswfile, source, env):
         self.dswfile = os.path.normpath(str(dswfile))
@@ -1451,7 +1489,7 @@ class _DSWGenerator(object):
 class _GenerateV7DSW(_DSWGenerator):
     """Generates a Solution file for MSVS .NET"""
     def __init__(self, dswfile, source, env):
-        _DSWGenerator.__init__(self, dswfile, source, env)
+        super().__init__(dswfile, source, env)
 
         self.file = None
         self.version = self.env['MSVS_VERSION']
@@ -1494,7 +1532,8 @@ class _GenerateV7DSW(_DSWGenerator):
                 config.platform = 'Win32'
 
             self.configs[variant] = config
-            print("Adding '" + self.name + ' - ' + config.variant + '|' + config.platform + "' to '" + str(dswfile) + "'")
+            # DEBUG
+            # print("Adding '" + self.name + ' - ' + config.variant + '|' + config.platform + "' to '" + str(dswfile) + "'")
 
         if 'variant' not in env:
             raise SCons.Errors.InternalError("You must specify a 'variant' argument (i.e. 'Debug' or " +\
@@ -1505,11 +1544,9 @@ class _GenerateV7DSW(_DSWGenerator):
             for variant in env['variant']:
                 AddConfig(self, variant)
 
-        self.platforms = []
-        for key in list(self.configs.keys()):
-            platform = self.configs[key].platform
-            if platform not in self.platforms:
-                self.platforms.append(platform)
+        seen = set()
+        self.platforms = [p.platform for p in self.configs.values()
+                          if not (p.platform in seen or seen.add(p.platform))]
 
         def GenerateProjectFilesInfo(self):
             for dspfile in self.dspfiles:
@@ -1567,7 +1604,10 @@ class _GenerateV7DSW(_DSWGenerator):
     def PrintSolution(self):
         """Writes a solution file"""
         self.file.write('Microsoft Visual Studio Solution File, Format Version %s\n' % self.versionstr)
-        if self.version_num >= 14.2:
+        if self.version_num >= 14.3:
+            # Visual Studio 2022 is considered to be version 17.
+            self.file.write('# Visual Studio 17\n')
+        elif self.version_num >= 14.2:
             # Visual Studio 2019 is considered to be version 16.
             self.file.write('# Visual Studio 16\n')
         elif self.version_num > 14.0:
@@ -2039,7 +2079,7 @@ def generate(env):
     env['MSVSCLEANCOM'] = '$MSVSSCONSCOM -c "$MSVSBUILDTARGET"'
 
     # Set-up ms tools paths for default version
-    msvc_setup_env_once(env)
+    msvc_setup_env_once(env, tool=tool_name)
 
     if 'MSVS_VERSION' in env:
         version_num, suite = msvs_parse_version(env['MSVS_VERSION'])
@@ -2047,17 +2087,17 @@ def generate(env):
         (version_num, suite) = (7.0, None) # guess at a default
     if 'MSVS' not in env:
         env['MSVS'] = {}
-    if (version_num < 7.0):
+    if version_num < 7.0:
         env['MSVS']['PROJECTSUFFIX']  = '.dsp'
         env['MSVS']['SOLUTIONSUFFIX'] = '.dsw'
-    elif (version_num < 10.0):
+    elif version_num < 10.0:
         env['MSVS']['PROJECTSUFFIX']  = '.vcproj'
         env['MSVS']['SOLUTIONSUFFIX'] = '.sln'
     else:
         env['MSVS']['PROJECTSUFFIX']  = '.vcxproj'
         env['MSVS']['SOLUTIONSUFFIX'] = '.sln'
 
-    if (version_num >= 10.0):
+    if version_num >= 10.0:
         env['MSVSENCODING'] = 'utf-8'
     else:
         env['MSVSENCODING'] = 'Windows-1252'
@@ -2069,7 +2109,7 @@ def generate(env):
     env['SCONS_HOME'] = os.environ.get('SCONS_HOME')
 
 def exists(env):
-    return msvc_exists(env)
+    return msvc_setup_env_tool(env, tool=tool_name)
 
 # Local Variables:
 # tab-width:4
