@@ -1,7 +1,7 @@
 __version__="4.4.0"
-__copyright__="Copyright (c) 2001 - 2022 The SCons Foundation"
+__copyright__="Copyright (c) 2001 - 2023 The SCons Foundation"
 __developer__="_reproducible"
-__date__="Sun, 07 Aug 2022 09:54:35 -0600"
+__date__="Mon, 30 Jan 2023 21:51:33 -0600"
 __buildsys__="_reproducible"
 __revision__=""
 __build__=""
